namespace Mail
{
	using System;
	using System.Collections;
	using System.IO;
	using System.Text;
	using System.Text.RegularExpressions;
	using System.Security.Cryptography;

	using Mail.Providers;

	/// <summary>
	///    <see cref="Mime"/>  <see cref="Message"/>. 
	///      .
	/// </summary>
	public class Common
	{
		internal const string DCRLF = "\r\n\r\n";
		internal const string CRLF = "\r\n";
		internal const char Colon = ':';
		internal const string MIMEE = "--";

		internal string _message;
		internal long _size;

		internal Provider _pop3;

		public Provider Parent
		{
			get
			{
				return _pop3;
			}			
		}

		/// <summary>
		///  sub-mimes.
		/// </summary>
		public Mime [] ChildMimes = new Mime[0];

		/// <summary>
		/// .
		/// </summary>
		internal Hashtable Headers = new Hashtable();

		/// <summary>
        ///     <see cref="Message"/>  <see cref="Mime"/>
		/// </summary>
		/// <remarks>
        ///       .
		/// </remarks>
		public Hashtable AllHeaders
		{
			get
			{
                return Headers;
			}
		}

		/// <summary>
		///  .
		/// </summary>
		public long GetSize()
		{
			return _size;			
		}
		/// <summary>
		///        .
		/// </summary>
		/// <param name="mes"> .</param>
		/// <returns></returns>
		internal string PreparedString(string mes)
		{
			string t;
			switch (TransferEncoding)
			{
				case "base64":
					t = DecodeMessage(mes, new FromBase64Transform());
					break;

				case "quoted-printable":
					t = DecodeMessage(mes, new FromQuotedPrintableTransform());
					break;

				default:
					t = mes;
					break;
			}

			return t;
		}
		/// <summary>
		///    .
		/// </summary>
		/// <returns></returns>
		internal string PreparedBody()
		{
			
			return PreparedString(_message);
		}
		/// <summary>
		///  .
		/// </summary>
		/// <param name="message"> .</param>
		/// <param name="transform"> .</param>
		/// <returns></returns>
		string DecodeMessage(string message, ICryptoTransform transform)
		{
			MemoryStream tf = new MemoryStream();
			CryptoStream cs = new CryptoStream(tf, transform, CryptoStreamMode.Write);
				
			//      
			Encoding enc = Encoding.ASCII;
			byte [] b = enc.GetBytes(message);
			cs.Write(b, 0, b.Length);

			cs.Close();
			string t = Utils.GetEncoding(Charset).GetString(tf.ToArray());
			tf.Close();
			return t;
		}
		/// <summary>
		/// .
		/// </summary>
		/// <param name="parent"> .</param>
		/// <param name="message">  .</param>
		internal Common(Provider parent, string message)
		{
			if (parent == null)
			{
				throw new ArgumentNullException("parent");
			}
			if (message == String.Empty)
			{
				throw new ArgumentException("empty string", "message");
			}

			int end = FillHeaders(message);
			_size = message.Length;

			//     
			_message = message.Substring(end);
			_pop3 = parent;
		}

		/// <summary>
        ///      .
		/// </summary>
		/// <returns></returns>
		internal int FillHeaders(string message)
		{
			int start = 0; //message.IndexOf(CRLF) + CRLF.Length; // 2 
			int headerend = message.IndexOf(DCRLF); 
			string headers = message.Substring(start, headerend - start);
			GetHeaders(headers);

			//   
			headerend += DCRLF.Length;

			return headerend;
		}

		/// <summary>
		///   <see cref="Mime"/> .
		/// </summary>
		/// <returns></returns>
		protected bool MultipartMixed()
		{
			string b = GetBoundary();
			if (b == String.Empty) return false;

			int s = _message.IndexOf(b);
			if (s < 0)
			{
				throw new ParseException("Can't find beginning MIME boundary: " + b);
			}

			ArrayList tmimes = new ArrayList();
			
			while(true)
			{
				s += b.Length + CRLF.Length;

				int e = _message.IndexOf(b, s);
				if (e < 0)
				{					
					if (_message.IndexOf(MIMEE, s - CRLF.Length, MIMEE.Length) < 0)
					{
						throw new ParseException(" MIME");
					}

					break;
				}
                			

				tmimes.Add(new Mime(_pop3, _message.Substring(s, e - s - CRLF.Length)));
				s = e;
			}

			ChildMimes = (Mime [])tmimes.ToArray(typeof(Mime));
            
			return true;
		}


		/// <summary>
		///    'boundary'  <see cref="ContentType"/>.
		/// </summary>
		/// <returns></returns>
		protected string GetBoundary()
		{			
			Regex r = new Regex("boundary=[\\\"]?([^\\r\\\"]+)");
			if (ContentType == null)
			{
                return String.Empty;
			}

			Match m = r.Match(ContentType);
			if (m.Success)
			{
				return "--" + m.Groups[1].ToString();
			}
			else
			{
				return String.Empty;
			}
		}



		/// <summary>
		///     .
		/// </summary>
		/// <param name="top"></param>
		void GetHeaders(string top)
		{			
			Regex line = new Regex(@"\r\n(?![\t\x20])");
			string [] col = line.Split(top);
			foreach (string s in col)
			{
				string [] fields = s.Split(new Char[] {':'}, 2);
				//				Console.WriteLine(fields[0] + "}={" + fields[1] + "}");
				if (fields.Length < 2) continue;
				fields[0] = fields[0].ToLower(); //    
				fields[1] = fields[1].TrimStart(' '); //   

				
				if (Headers.ContainsKey(fields[0]))
				{
					object oldv = Headers[fields[0]];
					ArrayList al = oldv as ArrayList;
					if (al == null)
					{
						al = new ArrayList();
						al.Add(oldv); 
						Headers[fields[0]] = al;
					} 

					al.Add(fields[1]);
				} 
				else 
				{
					Headers.Add(fields[0].ToLower(), fields[1]);
				}
			}
		}

		#region Common headers
		public string Charset
		{
			get
			{
				Regex r = new Regex(@"charset=[""'\s]([^""'\s]+)");
				Match m = r.Match(ContentType);
				if (m.Success)
					return m.Groups[1].Value;
				else
					return "";
			}
		}

		protected string TransferEncoding
		{
			get
			{
				return ((string)Headers["content-transfer-encoding"]).ToLower();
			}
		}

		/// <summary>
		///    <see cref="Mime"/>   <see cref="Message"/>.
		/// </summary>		
		public string ContentType
		{
			get
			{
				return (string)Headers["content-type"];
			}
		}
		#endregion
	}
}