//#define _DEBUG

namespace Mail
{
	using System;
	using System.Diagnostics;
	using System.Text;
	using System.IO;
	using System.Text.RegularExpressions;
	using System.Security.Cryptography;

	using Mail.Providers;

	public class Mime : Common
	{
		#region DEBUG
		FileStream GetDebugStream()
		{
			return new FileStream(@"C:\trace_mimes.log", FileMode.Append);
		}
		[Conditional("_DEBUG")]
		void Trace(string str)
		{
			FileStream fs = GetDebugStream();
			byte [] b = Encoding.ASCII.GetBytes(str);
			fs.Write(b, 0, b.Length);
			fs.Close();
		}
		#endregion

		internal AttachDescriptor _attach = null;

		/// <summary>
		/// .
		/// </summary>
		/// <param name="pm">.</param>
		/// <param name="message"> .</param>
		internal Mime(Provider pm, string message) : base(pm, message)
		{			
            //  MIME   MultipartMixed,        
			if (!MultipartMixed())
			{
				FindAttachment();
			}
		}
		/// <summary>
		///    .
		/// </summary>
		void FindAttachment()
		{
			//   ,  
			if (ContentDisposition == null) 
			{
                return;
			}

			string [] cd = ContentDisposition.Split(new char [] {';'}, 2);
			switch(cd[0].ToLower())
			{
                case "attachment":
					ExtractAttachment(cd[1]);
					break;
                
				case "inline":
					throw new CoreException(" ");

				default:
					throw new ParseException(" ContentDisposition:" + cd[0]);
			}
		}
		/// <summary>
		///    .
		/// </summary>
		/// <param name="filename"></param>
		/// <returns></returns>
		string GetAttachmentFilename(string filename)
		{
			Regex r = new Regex("filename=[\\\"]?([^\\r\\\"]+)");
			Match m = r.Match(filename);
			if (!m.Success)
			{
                return String.Empty;
			}			

			return Utils.WordDecoder(m.Groups[1].ToString());
		}
		/// <summary>
		///     .
		/// </summary>
		/// <param name="filename">  .</param>
		void ExtractAttachment(string filename)
		{
			_attach = new AttachDescriptor(GetAttachmentFilename(filename), _pop3.TempDirectory);

			switch (TransferEncoding)
			{
				case "base64":
					_attach.DecodeFile(_message, new FromBase64Transform());
					break;

				case "quoted-printable":
					_attach.DecodeFile(_message, new FromQuotedPrintableTransform());
					break;

				default:
                    Debug.WriteLine("  .");
					break;
			}
		}

		#region Common headers
		/// <summary>
		///    content-disposition,   .
		/// </summary>
		string ContentDisposition
		{
			get
			{
				return (string)Headers["content-disposition"];
			}
		}
		#endregion
	}
}
